use embedded_hal::digital::v2::InputPin;

pub struct Button<'a, P> {
    pin: &'a P,
    state: bool,
    is_pullup: bool,
}

#[derive(PartialEq)]
pub enum Event {
    PressDown,
    PressUp,
    Pressed,
    None,
}

impl<'a, P: InputPin> Button<'a, P> {
    pub fn new(pin: &'a P, is_pullup: bool) -> Self {
        let mut state = pin.is_high().map_err(|_| ()).unwrap();
        if is_pullup {
            state = !state;
        }

        Button {
            pin,
            state,
            is_pullup,
        }
    }

    fn read_pin(&self) -> bool {
        let mut inc = 0;

        for _ in 0..100 {
            let mut state = self.pin.is_high().map_err(|_| ()).unwrap();
            if self.is_pullup {
                state = !state;
            }
            if state {
                inc += 1;
            }
        }

        inc > 70
    }

    pub fn update(&mut self) -> Event {
        let actual_state = self.read_pin();

        match (self.state, actual_state) {
            (false, false) => Event::None,
            (true, false) => {
                self.state = !self.state;
                Event::PressUp
            }
            (false, true) => {
                self.state = !self.state;
                Event::PressDown
            }
            (true, true) => Event::Pressed,
        }
    }
}
